/*
 * Linux_OSCrashDumpCapture.c
 *
 * (C) Copyright IBM Corp. 2009
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author:       Sachin Sant <sachinp@in.ibm.com>
 *
 * Description:
 * Implements resource access functionality for the
 * class Linux_OSCrash
 */

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include "OSBase_Common.h"

#include "Linux_OSCrash.h"

int linux_distro = 0;
#define REDHAT 1
#define SUSE 2
#define UNKNOWN 3

/*
 * Find out which distro (REDHAT/SUSE) is running ?
 */
static int get_linux_distro_name(void)
{
	char **tmpbuf = NULL;
	int rc = 0;

	_OSBASE_TRACE(3,(" --- get_linux_distro_name() called"));

	rc = runcommand( "cat /etc/redhat-release 2>/dev/null",
			NULL,
			&tmpbuf,
			NULL);
	if (rc == 0 ) {
		if (tmpbuf[0] != "") {
			freeresultbuf(tmpbuf);
			return REDHAT;
		}
	}
	if (tmpbuf)
		freeresultbuf(tmpbuf);

	rc = runcommand( "cat /etc/SuSE-release 2>/dev/null",
			NULL,
			&tmpbuf,
			NULL);
	if (rc == 0 ) {
		if (tmpbuf[0] != "") {
			freeresultbuf(tmpbuf);
			return SUSE;
		}
	}
	if (tmpbuf)
		freeresultbuf(tmpbuf);

	_OSBASE_TRACE(3,(" --- get_linux_distro_name() exited"));

	return UNKNOWN;
}

/*
 * Record Crash Dump Capture Date and Time.
 */
static char *get_os_crashdumpcapturedatetime(void)
{
	char **tmpbuf = NULL;
	char *cdatetime_str = NULL;
	char *ptr = NULL;
	int rc = 0, buflen = 0;

	_OSBASE_TRACE(3,(" --- get_os_crashdumpcapturedatetime() called"));

	if (linux_distro == SUSE)
		rc = runcommand( "ls -ct /var/log/dump/ | sed -e '{2,$ d}'",
				NULL,
				&tmpbuf,
				NULL);
	else if (linux_distro == REDHAT)
		rc = runcommand( "ls -ct /var/crash/ | sed -e '{2,$ d}'",
				NULL,
				&tmpbuf,
				NULL);
	else
		rc = -1;

	if (rc == 0 && tmpbuf[0] != NULL) {
		buflen = strlen(tmpbuf[0]) + 1;
		cdatetime_str = (char *)calloc(1, buflen);
		strncpy(cdatetime_str, tmpbuf[0], buflen);
		if ((ptr = strchr(cdatetime_str, '\n')))
			*ptr = '\0';
	} else {
		cdatetime_str = (char *)calloc(1, 14);
		strncpy(cdatetime_str, "Not available", 14);
	}

	if (tmpbuf)
		freeresultbuf(tmpbuf);
		
	_OSBASE_TRACE(3,(" --- get_os_crashdumpcapturedatetime() exited"));

	return cdatetime_str;
}

/*
 * Get Crash Dump File Location.
 */
static char *get_os_crashdumpfilelocation(void)
{
	char **tmpbuf = NULL;
	char *cdumpfile_str = NULL;
	int rc = 0, buflen = 0;

	_OSBASE_TRACE(3,(" --- get_os_crashdumpfilelocation() called"));

	if (linux_distro == SUSE)
		rc = runcommand( "ls -ct /var/log/dump/ | sed -e '{2,$ d}'",
				NULL,
				&tmpbuf,
				NULL);
	else if (linux_distro == REDHAT)
		rc = runcommand( "ls -ct /var/crash/ | sed -e '{2,$ d}'",
				NULL,
				&tmpbuf,
				NULL);
	else
		rc = -1;

	if (rc == 0 && tmpbuf[0] != NULL) {
		char * ptr = strchr(tmpbuf[0], '\n');
		if (ptr)
			*ptr = '\0';
		buflen = strlen(tmpbuf[0]);
		if (linux_distro == SUSE) {
			buflen += strlen("/var/log/dump/");
			/* space for '/' char and '\0' */
			buflen += 2;
			cdumpfile_str = (char *)calloc( 1, buflen);
			strcpy(cdumpfile_str, "/var/log/dump/");
			strcat(cdumpfile_str, tmpbuf[0]);
			strcat(cdumpfile_str, "/");
		}
		if (linux_distro == REDHAT) {
			buflen += strlen("/var/crash/");
			/* space for '/' char and '\0' */
			buflen += 2;
			cdumpfile_str = (char *)calloc(1, buflen);
			strcpy(cdumpfile_str, "/var/crash/");
			strcat(cdumpfile_str, tmpbuf[0]);
			strcat(cdumpfile_str, "/");
		}
	} else {
		cdumpfile_str = (char *)calloc(1, 14);
		strncpy(cdumpfile_str, "Not available", 14);
	}
	
	if (tmpbuf)
		freeresultbuf(tmpbuf);

	_OSBASE_TRACE(3,(" --- get_os_crashdumpfilelocation() exited"));

	return cdumpfile_str;
}

/*
 * Get Crash Dump Size.
 */
static unsigned long long get_os_crashdumpsize(char *cfileloc_str,
					char *cfilename_str)
{
	char **tmpbuf = NULL;
	char *tmp_str = NULL;
	unsigned long long cdumpsize = 0;
	int rc = 0, buflen = 0;

	_OSBASE_TRACE(3,(" --- get_os_crashdumpsize() called"));

	if (cfileloc_str
		&& (strcmp(cfileloc_str, "Not available") != 0) 
		&& cfilename_str
		&& (strcmp(cfilename_str, "Not available") != 0)) {
		buflen = strlen(cfileloc_str) + strlen(cfilename_str);
		tmp_str = calloc(1, buflen + 28);
		strcpy(tmp_str, "ls -l ");
		strncat(tmp_str, cfileloc_str, strlen(cfileloc_str));
		strncat(tmp_str, cfilename_str, strlen(cfilename_str));
		strcat(tmp_str, " |awk '{ print $5 }'");
		rc = runcommand(  tmp_str, NULL, &tmpbuf, NULL);
	} else
		rc = -1;

	if (rc == 0 && tmpbuf[0] != NULL)
		cdumpsize = strtoull(tmpbuf[0], NULL, 0);
	else
		cdumpsize = 0;

	if (tmp_str)
		free(tmp_str);
	if(tmpbuf)
		freeresultbuf(tmpbuf);

	_OSBASE_TRACE(3,(" --- get_os_crashdumpsize() exited"));

	return cdumpsize;
}

/* Returns a structure, which contains necessary information about Crash Dump 
 * Capture Event
 */

int get_Linux_CrashDumpCapture_data( struct oscrash_data **ptr )
{
	_OSBASE_TRACE(3,(" --- get_Linux_CrashDumpCapture_data() called"));

	if (*ptr == NULL ) {
		_OSBASE_TRACE(3,
			(" -- Invalid Argument (ptr=NULL) passed"));
		_OSBASE_TRACE(3,
			(" --- get_Linux_CrashDumpCapture_data() exited"));
		return -1;
	}

	linux_distro = get_linux_distro_name();

	(*ptr)->dumpFileLocation = get_os_crashdumpfilelocation();

	if (strcmp((*ptr)->dumpFileLocation, "Not available") == 0) {
		(*ptr)->dumpFileName = (char *)calloc(1, 14);
		strncpy((*ptr)->dumpFileName, "Not available", 14);
	} else {
		(*ptr)->dumpFileName = (char *)calloc(1, 7);
		strncpy((*ptr)->dumpFileName, "vmcore", 7);
	}

	(*ptr)->dumpCaptureDateTime = get_os_crashdumpcapturedatetime();
	(*ptr)->dumpFileSize = get_os_crashdumpsize(
					(*ptr)->dumpFileLocation,
					(*ptr)->dumpFileName);

	_OSBASE_TRACE(4,("================================================"));
	_OSBASE_TRACE(4,("     Dumpfile Location : %s",
						(*ptr)->dumpFileLocation));
	_OSBASE_TRACE(4,("         Dumpfile Name : %s", (*ptr)->dumpFileName));
	_OSBASE_TRACE(4,("Dump Capture Date time : %s",
						(*ptr)->dumpCaptureDateTime));
	_OSBASE_TRACE(4,("         Dumpfile size : %Ld", (*ptr)->dumpFileSize));
	_OSBASE_TRACE(4,("================================================"));

	_OSBASE_TRACE(3,(" --- get_Linux_CrashDumpCapture_data() exited"));

	return 0;
}
